function [qfit,Ifit,parameters,varargout]=saxsfit(q,I,funcname,initialguess,varargin)
%Usage: [qfit,Ifit,parameters]=saxsfit(q,I,funcname,initialguess,varargin)
%input: q, I, functionname, initialguess, and a series of parameter-value
%pairs, amongst which optionally:
%'lowerlimit' - lower constraint to fit
%'upperlimit' - upper constraint to fit
% 'Ierr' - error values for the intensity
% 'interpolationnumber' - interpolated intensity using #n sampling points.
% if #n is fractional, it is considered to be a fraction of the data points
% 'qlims' - limits of the fit in the q range
% 'separateintensity' - a two-element integer vector indicating which
% parameters are the intensity scaling factor and background. These will
% then be optimised separately, to reduce the degrees of freedom of the
% fitting function and speeding it up that way.

%input checking
if size(q)~=size(I)
    error('Size of q must be equal to size of I!');
end
if ~ischar(funcname)
    error('function name must be a string!')
end
if isempty(initialguess)
    error('no initial guess provided!')
end
kansas=size(q);
fitfunc=str2func(funcname);
x0=initialguess;
fitpar.tol=1e-5;
in=0;

if min(size(q)~=1) %multidimensional matrices, multiple measurements.
    Ierr=sqrt(max(min(min(I+max(max(I)).*(I==0))),abs(I))); % just making sure we do not get imaginary numbers.
else
    Ierr=sqrt(max(min(I+max(I).*(I==0)),abs(I))); % just making sure we do not get imaginary numbers.
end
fitll=-inf.*ones(size(x0));
fitul=inf.*ones(size(x0));
qlims=[min(q) max(q)];
rcor=0;
if nargin>3 %process inputs
    vi=0;
    %varargin=varargin{1};
    npv=length(varargin);
    while vi<npv
        vi=vi+1;
        switch varargin{vi}
            case 'lowerlimit'
                vi=vi+1;
                fitll=varargin{vi};
            case 'upperlimit'
                vi=vi+1;
                fitul=varargin{vi};
            case 'Ierr'
                vi=vi+1;
                Ierr=varargin{vi};
                if size(Ierr)~=size(I);
                    error('Ierr must be the same size as I!')
                end
            case 'interpolationnumber'
                vi=vi+1;
                in=varargin{vi};
                if in>1;
                    in=round(in);
                else %in=fraction of original number of datapoints
                    if min(size(q)~=1) %multidimensional matrices, multiple measurements.
                        %determine the direction in which the datasets are present; 1=rows (i measurement per row),
                        %2=columns (one measurement per column). We can do this by analysing
                        %the direction of q
                        if max(q(1,:))>max(q(:,1)) %data in rows
                            direction=1;
                        else
                            direction=2;
                        end
                    end
                    in=round(size(q,direction)/in); %check this!
                end
            case 'qlims'
                vi=vi+1;
                qlims=varargin{vi};
            case 'separateintensity'
                vi=vi+1;
                si.param=varargin{vi};
                si.ll=fitll(si.param);
                si.ul=fitul(si.param);
                si.ig=initialguess(si.param);
            case 'reversecorrelation'
                vi=vi+1;
                if varargin{vi}&&min(size(q)~=1); %needs multiple measurements to work
                    rcor=1;
                    if max(q(1,:))>max(q(:,1)) %data in rows
                        direction=1;
                        q=q(end:-1:1,:);
                        I=I(end:-1:1,:);
                    else
                        direction=2;
                        q=q(:,end:-1:1);
                        I=I(:,end:-1:1);
                    end

                end

            otherwise
                vi=vi+1;
                warning('inputprocessing:invalidparameter','parameter %s is not one of the acceptable parameters. Check your usage',varargin{vi-1});
        end
    end

end


if min(size(q)~=1) %multidimensional matrices, multiple measurements.
    %determine the direction in which the datasets are present; 1=rows (i measurement per row),
    %2=columns (one measurement per column). We can do this by analysing
    %the direction of q
    if max(q(1,:))>max(q(:,1)) %data in rows
        direction=1;
    else
        direction=2;
        q=q';
        I=I';
        Ierr=Ierr';
        %remember to transform back at the end of the exercise
    end
    Ifit=zeros(size(I,1),length(find(q(1,:)>=min(qlims)&q(1,:)<=max(qlims))));
    qfit=zeros(size(I,1),length(find(q(1,:)>=min(qlims)&q(1,:)<=max(qlims))));
    for ii=1:size(I,1);

        %condition q and I

        %apply limits
        qf=q(ii,find(q(ii,:)>=min(qlims)&q(ii,:)<=max(qlims)));
        Id=I(ii,find(q(ii,:)>=min(qlims)&q(ii,:)<=max(qlims)));
        Ie=Ierr(ii,find(q(ii,:)>=min(qlims)&q(ii,:)<=max(qlims)));
        %end

        %conditioning of q in the case of interpolation
        if in~=0
            qo=qf; %original q
            qmin=min(qo);
            qmax=max(qo);
            qf=qmin:(qmax-qmin)/(in-1):qmax; %reshape q into new spanning range
            Id=interp1(qo,Id,qf);
            Ie=interp1(qo,Ie,qf);
        end




        %do the actual fitting, reusing the optimised values as starting
        %parameters for the next experiment

        % progressbar(ii/size(I,1))
        %for "safety", fit using the initial guess and the previous
        %optimisation. Then check which is lower.
        Fitval=inf;
        noescape=1;
        inumber=1;

        %fit using initial guess
        x0=initialguess;
        while noescape && inumber<10
            display(sprintf('invocation number %s',num2str(inumber)));
            if exist('si','var'); %separate intensity optimisation
                meritfunc=@(x) merit(qf,Id,Ie,funcname,x,si);
                fitll(si.param)=[1 0];
                fitul(si.param)=[1 0];
                x0(si.param)=[1 0];
            else
                meritfunc=@(x) merit(qf,Id,Ie,funcname,x);
            end
            [Fitparam,newFitval,exitflag,output]=...
                fminsearchbnd(meritfunc,x0,fitll,fitul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));

            %[Fitparam,newFitval,exitflag,output]=...
            %fminsearchbnd(meritfunc,x0,xl,xu,optimset('MaxFunEvals',fitpar.numiter,'MaxIter',fitpar.numiter,'Tolx',fitpar.tol));
            if (newFitval/Fitval)<(1+fitpar.tol) && (newFitval/Fitval)>(1-fitpar.tol) %falls within tolerances
                noescape=0;

                csq(ii)=newFitval;
            else
                inumber=inumber+1;
                Fitval=newFitval;
                x0=Fitparam;
            end
        end
        parameter_initialguess=Fitparam;
        Fitval_initialguess=newFitval;

        %fit using previous optimisations
        if ii>1 %not for the first fit, of course
            x0=parameters(ii-1,:);
            while noescape && inumber<10
                display(sprintf('invocation number %s',num2str(inumber)));
                if exist('si','var'); %separate intensity optimisation
                    meritfunc=@(x) merit(qf,Id,Ie,funcname,x,si);
                    fitll(si.param)=[1 0];
                    fitul(si.param)=[1 0];
                    x0(si.param)=[1 0];
                else
                    meritfunc=@(x) merit(qf,Id,Ie,funcname,x);
                end
                [Fitparam,newFitval,exitflag,output]=...
                    fminsearchbnd(meritfunc,x0,fitll,fitul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));

                %[Fitparam,newFitval,exitflag,output]=...
                %fminsearchbnd(meritfunc,x0,xl,xu,optimset('MaxFunEvals',fitpar.numiter,'MaxIter',fitpar.numiter,'Tolx',fitpar.tol));
                if (newFitval/Fitval)<(1+fitpar.tol) && (newFitval/Fitval)>(1-fitpar.tol) %falls within tolerances
                    noescape=0;
                    csq(ii)=newFitval;
                else
                    inumber=inumber+1;
                    Fitval=newFitval;
                    x0=Fitparam;
                end
            end
        end
        parameter_prevopt=Fitparam;
        Fitval_prevopt=newFitval;
        %save the best result in parameters
        if Fitval_initialguess<Fitval_prevopt
            parameters(ii,:)=parameter_initialguess;
        else
            parameters(ii,:)=parameter_prevopt;
        end

        If=fitfunc(qf,parameters(ii,:));
        tic
        if exist('si','var') %re-optimise intensity scaling
            sifunc=@(x) optI(Id,If,Ie,x);
            [Fitparam,csq(ii),exitflag,output]=...
                fminsearchbnd(sifunc,si.ig,si.ll,si.ul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));
            parameters(ii,si.param)=Fitparam;
        end
        toc
        %         if exist('si','var') %re-optimise intensity scaling
        %             Ifit=fitfunc(qf,parameters);
        %             sifunc=@(x) optI(Id,Ifit,Ie,x);
        %             [Fitparam,cs,exitflag,output]=...
        %                 fminsearchbnd(sifunc,si.ig,si.ll,si.ul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));
        %             parameters(si.param)=Fitparam;
        %         end

        %to be placed after the fitting:
        if in~=0
            %Ifit=interp1(qf,If,qo);
            qfit(ii,:)=qo;
        else
            qfit(ii,:)=qf;
            %Ifit=If;
        end

        Ifit(ii,:)=fitfunc(qfit(ii,:),parameters(ii,:));

        %         %to be placed after the fitting:
        %         if in~=0
        %             Ifit(ii,:)=interp1(qf,If,qo);
        %             qfit(ii,:)=qo;
        %         else
        %             qfit=qf;
        %             Ifit(ii,:)=If;
        %         end

    end
    csqr=csq;
    if direction==2
        Ifit=Ifit';
        qfit=qfit';
    end
else
    Fitval=inf;
    noescape=1;
    inumber=1;
    %apply limits
    qf=q(find(q>=min(qlims)&q<=max(qlims)));
    Id=I(find(q>=min(qlims)&q<=max(qlims)));
    Ie=Ierr(find(q>=min(qlims)&q<=max(qlims)));
    %end

    %conditioning of q in the case of interpolation
    if in~=0
        qo=qf; %original q
        qmin=min(qo);
        qmax=max(qo);
        qf=qmin:(qmax-qmin)/(in-1):qmax; %reshape q into new spanning range
        Id=interp1(qo,Id,qf);
        Ie=interp1(qo,Ie,qf);
    end

    %fit using initial guess
    x0=initialguess;
    while noescape && inumber<10
        display(sprintf('invocation number %s',num2str(inumber)));

        if exist('si','var'); %separate intensity optimisation
            meritfunc=@(x) merit(qf,Id,Ie,funcname,x,si);
            fitll(si.param)=[1 0];
            fitul(si.param)=[1 0];
            x0(si.param)=[1 0];
        else
            meritfunc=@(x) merit(qf,Id,Ie,funcname,x);
        end

        [Fitparam,newFitval,exitflag,output]=...
            fminsearchbnd(meritfunc,x0,fitll,fitul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));

        %[Fitparam,newFitval,exitflag,output]=...
        %fminsearchbnd(meritfunc,x0,xl,xu,optimset('MaxFunEvals',fitpar.numiter,'MaxIter',fitpar.numiter,'Tolx',fitpar.tol));
        if (newFitval/Fitval)<(1+fitpar.tol) && (newFitval/Fitval)>(1-fitpar.tol) %falls within tolerances
            noescape=0;
            csq=newFitval;
        else
            inumber=inumber+1;
            Fitval=newFitval;
            x0=Fitparam;
                        csq=newFitval;

        end
    end
    parameters=Fitparam;
    if exist('si','var') %re-optimise intensity scaling
        Ifit=fitfunc(qf,parameters);
        sifunc=@(x) optI(Id,Ifit,Ie,x);
        [Fitparam,csq,exitflag,output]=...
            fminsearchbnd(sifunc,si.ig,si.ll,si.ul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));
        parameters(si.param)=Fitparam;
    end

    %to be placed after the fitting:
    if in~=0
        %Ifit=interp1(qf,If,qo);
        qfit=qo;
    else
        qfit=qf;
        %Ifit=If;
    end

    Ifit=fitfunc(qfit,parameters);
    csqr=csq;
end
if nargout==4;varargout{1}=csqr;end
if rcor&&min(size(q)~=1); %needs multiple measurements to work
    if direction==1 %data in rows
        qfit=qfit(end:-1:1,:);
        Ifit=Ifit(end:-1:1,:);

    elseif direction==2;
        qfit=qfit(:,end:-1:1);
        Ifit=Ifit(:,end:-1:1);
    end
    parameters=parameters(end:-1:1,:);

end


end

function cs=merit(q,I,Ierr,funcname,fitvalues,varargin)
%optional argument: si matrix for separate intensity (and background)
%optimisation
if nargin==6 %separate intensities
    si=varargin{1};
    fitfunc=str2func(funcname);
    Icalc=fitfunc(q,fitvalues);

    %     cm=cov(I,Icalc);
    %     sc=cm(1)/cm(2);
    %     meani=mean(I);
    %     mini=min(I);
    %     bgnd=min(mini,meani-sc*mean(Icalc));
    %     cs=sum(((I-Icalc)./Ierr).^2)/(length(I)-length(fitvalues)); %weighted fitting
    %

    sifunc=@(x) optI(I,Icalc,Ierr,x);
    [Fitparam,cs,exitflag,output]=...
        fminsearchbnd(sifunc,si.ig,si.ll,si.ul,optimset('MaxFunEvals',100000,'MaxIter',100000,'Tolx',1e-5));
    %display(cs);
else

    fitfunc=str2func(funcname);
    Icalc=fitfunc(q,fitvalues);

    % if isempty(Ierr)
    %     cs=(sum((I-Icalc).^2))/(length(I)-length(fitvalues)); %unweighted
    % else
    cs=sum(((I-Icalc)./Ierr).^2)/(length(I)-length(fitvalues)); %weighted fitting
    %cs=sum(((log10(I)-log10(Icalc))./Ierr).^2)/(length(I)-length(fitvalues)); %weighted fitting
    % end
    %display(cs);

end
if isnan(cs)||cs<=0
    warning('Chi Squared is assuming impossible values, best check your fitting function behaviour for these values:')

    if exist('Fitparam','var')
        display(num2str(Fitparam));
    else
        display(num2str(fitvalues));
    end
end
end

function cs=optI(I,Icalc,Ierr,fitvalues)
Isc=fitvalues(1);
bgnd=fitvalues(2);
cs=sum(((I-(Isc.*Icalc+bgnd))./Ierr).^2)/(length(I)-length(fitvalues)); %weighted fitting
%cs=sum(((log10(I)-log10((Isc.*Icalc+bgnd)))./Ierr).^2)/(length(I)-length(fitvalues)); %log-weighted fitting
end